﻿/* ETBSaveAsOld.jsx ------ UTF-8 -----------------------------------------------------------------
              Save FM document or book as old version
Documentation www.daube.ch/docu/fmaker73.html
Comment       To be placed in Startup folder of user area
              For a list of DDD used Esc shortcuts see E:\_DDDprojects\FM-GUI-modifications\ESC-sequences\FM-ESC-sequences.fm
Reference     Based on SaveBookAsOldVersion 2013-09-04 by Carmen Publishing Inc. (www.frameexpert.com)
              Traduction française avec l'aide de Babylon et Deepl par Klaus Daube
Attention     Code must be UTF-8. With Windows cp no umlauts etc. are allowed in menu items.
Copyleft      2017 Klaus Daube, Zürich, Switzerland, klaus@daube.ch
History       v1.0  2016-05-24 Klaus Daube, Docu+Design Daube, klaus@daube.ch
              v1.1  2016-06-03 Save as MIF added.
                    It is not possible to save to another MIF version as the currnt FM
                    or to MIF-7 (FV_SaveFmtInterchange70)
                    Only Constants.FV_SaveFmtInterchange is valid in our case
              v1.2  2017-04-25 Adapted for FM-14 and integration into ETB
              v1.3  2017-06-30 Revert to dialogue, Inform user about current document location (for ordinary save)
              v1.4  2018-01-16 New command names with underscore
              v1.5  2018-04-20 checkbox for overwriting current with warning
              v1.6  2018-06-29 Hint about Rick Quatro's script
              v1.9  2023-03-27 Added support for FM-16, 17 same v to ETB14up
                               It is not possible to save MIF as specific as the FM version
                               (► save creates FV_BadSaveScriptValue in return parms)
*/ ; // ===========================================================================================
//@target framemaker

var gsETBsavO_AppVersion = "v1.7";                // version of this script

var gsETBsavO_title     = { en: "Save book or document in older version",
                            de: "Buch oder dokument in älterer version speichern",
                            fr: "Enregistrer le livre ou document en version plus ancienne"};
var gsETBsavO_NoDoc     = { en: "No book or document active",
                            de: "Kein buch oder dokument aktiv",
                            fr: "Aucun livre ou document actif"};
var gsETBsavO_Target    = { en: "Target folder is required.",
                            de: "Zielverzeichnis is notwendig",
                            fr: "Dossier cible est requise"};
var gsETBsavO_NoTarget  = { en: "Target folder does not exist",
                            de: "Zielverzeichnis nicht vorhanden",
                            fr: "Le dossier cible n'existe pas"};
var gsETBsavO_SelTarget = { en: "Please select a target folder",
                            de: "Bitte zielverzeichnis auswählen",
                            fr: "Veuillez sélectionner un dossier ciblee"};
var gsETBsavO_UnknownFM = { en: "Current FM version is not 10 ... 17",
                            de: "Aktuelle FM version ist nicht 10 ... 17",
                            fr: "Version FM actuelle n'est pas 10 ... 17"};
var gsETBsavO_Warning   = { en: "This will overwrite the current version! Do your really want this?",
                            de: "Dies überschreibt die aktuelle version! Wollen Sie das wirklich?",
                            fr: "Ceci remplace la version actuelle ! C'est ce que tu veux vraiment ?"};
var gsETBsavO_Success   = { en: "Document or book saved\n\nAttention: Be aware that the current directory is still the Target Folder!",
                            de: "Dokument oder buch gespeichert\n\nAchtung: Das aktuelle verzeichnis ist immer noch das Zielverzeichnis!",
                            fr: "Document ou livre enregistré\n\nAttention : Notez que le répertoire courant est toujours le dossier cible !"};

var goETBsavO_ActiveBook, goETBsavO_ActiveDoc;
var gsETBsavO_FMversion;                          // FM-8 ... FM-17
var goETBsavO_Folder;                             // target folder
var gbETBsavO_MIF;                                // true for handling MIF
var giETBsavO_FileType;                           // Constants.FV_SaveFmtBinary... | ...FV_SaveFmtInterchange

SwitchUILanguage ();
//DialogSaveOld();                                // un-comment for debugging
SetUpMenu();                                      // comment for debugging

function DialogSaveOld() { // =====================================================================
  if (!app.ActiveBook.ObjectValid() && !app.ActiveDoc.ObjectValid()) {
    alert ("ETBSaveAsOld: " + localize (gsETBsavO_NoDoc));
    return;
  }
  var stVersion = { en: "FM target version",
                    de: "FM Zielversion",
                    fr: "FM version cible"};
  var stFolder  = { en: "Target folder",
                    de: "Zielverzeichnis",
                    fr: "Dossier cible"};
  var stAdvice  = { en: "Enter folder name or use Browse button",
                    de: "Verzeichnis eingeben oder Schaltfläche Durchsuchen benutzen",
                    fr: "Entrez le nom du dossier ou utilisez le bouton Parcourir"};
  var sbBrowse  = { en: "Browse",
                    de: "Durchsuchen",
                    fr: "Parcourir"};
  var sbSaveFM  = { en: "Save as FM",
                    de: "Als FM sichern",
                    fr: "Enrgistrer comme FM"};
  var sbSaveMIF = { en: "Save as MIF",
                    de: "Als MIF sichern",
                    fr: "Enrgistrer comme  MIF"};
  var scCurrent = { en: "Save to current location",
                    de: "An den aktuellen Ort speichern ",
                    fr: "Enrgistrer comme  MIF"};
  var sbHelp    = { en: "Help",
                    de: "Hilfe (en)",
                    fr: "Aide (en)"};
  var sbCancel  = { en: "Cancel",
                    de: "Abbrechen",
                    fr: "Annuller"};
  var sbClose   = { en: "OK, close",
                    de: "OK, schliessen",
                    fr: "OK, conclure"};
  var stHint    = { en: "This script saves all files to the same directory.\nMIF is always saved in the same version as the current FM - not in the version selected for the target.\nRick Quatro's ConvertDocs script saves the converted files to the same directory as the source comes from. You may replace this script in the StrartUp folder by Rick's. See www.frameexpert.com/store ",
                    de: "Dieses Skript speichert die konvertierten Dateien in nur einem Verzeichnis.\nMIF wird immer in derselben Version wie der aktuelle FrameMaker gespeichert - nicht in der ausgewählten Zielversion.\nRick Quatro's ConvertDocs speichert die konvertierten Dateien im entsprechenden Quellverzeichnis. Dieses Skript im StartUp Verzeichnis kann gegen Rick's ausgetauscht werden. Siehe www.framexpert.com/store ",
                    fr: "Ce script enregistre tous les fichiers dans le même répertoire.\nLa MIF est toujours sauvegardée dans la même version que la FM actuelle - et non dans la version sélectionnée pour la cible.\nLe script ConvertDocs de Rick Quatro enregistre les fichiers convertis dans le même répertoire que la source. Vous pouvez remplacer ce script dans le dossier StrartUp par Rick's. Voir www.framexpert.com/store "};

  wSaveOld = new Window('dialog', localize(gsETBsavO_title) + "  " + gsETBsavO_AppVersion, undefined);
  wSaveOld.alignChildren = "left";
	wSaveOld.g1 = wSaveOld.add('group',undefined);
  	wSaveOld.g1.st1 = wSaveOld.g1.add('statictext',undefined,localize(stVersion));
  	wSaveOld.g1.rb = wSaveOld.g1.add('group',undefined);
    	wSaveOld.g1.rb.rb8  = wSaveOld.g1.rb.add('radiobutton',undefined,"FM-08");
    	wSaveOld.g1.rb.rb9  = wSaveOld.g1.rb.add('radiobutton',undefined,"FM-09");
    	wSaveOld.g1.rb.rb10 = wSaveOld.g1.rb.add('radiobutton',undefined,"FM-10");
    	wSaveOld.g1.rb.rb11 = wSaveOld.g1.rb.add('radiobutton',undefined,"FM-11");
    	wSaveOld.g1.rb.rb12 = wSaveOld.g1.rb.add('radiobutton',undefined,"FM-12");
    	wSaveOld.g1.rb.rb13 = wSaveOld.g1.rb.add('radiobutton',undefined,"FM-13");
    	wSaveOld.g1.rb.rb14 = wSaveOld.g1.rb.add('radiobutton',undefined,"FM-14");
    	wSaveOld.g1.rb.rb15 = wSaveOld.g1.rb.add('radiobutton',undefined,"FM-15");
    	wSaveOld.g1.rb.rb16 = wSaveOld.g1.rb.add('radiobutton',undefined,"FM-16");
    	wSaveOld.g1.rb.rb17 = wSaveOld.g1.rb.add('radiobutton',undefined,"FM-17");
     wSaveOld.rbtxt = wSaveOld.add('statictext',undefined,"[2015]         [2017]        [2019]         [2020]        [2022]");
//   wSaveOld.rbtxt.bounds = {x:400, y:30, width:270, height:15};
     wSaveOld.rbtxt.indent = 400
       wSaveOld.g2 = wSaveOld.add('group',undefined);
  	wSaveOld.g2.st2        = wSaveOld.g2.add('statictext',undefined,localize(stFolder));
  	wSaveOld.g2.folderName = wSaveOld.g2.add('edittext',undefined,localize(stAdvice));
  	wSaveOld.g2.folderName.preferredSize.width = 340;;
  	wSaveOld.g2.bBrowse    = wSaveOld.g2.add('button',undefined,localize(sbBrowse));
	wSaveOld.g3 = wSaveOld.add('group',undefined);
  	wSaveOld.g3.alignChildren = ['left',' '];
  	wSaveOld.g3.cCurrent = wSaveOld.g3.add("checkbox", undefined,localize(scCurrent));
  	wSaveOld.g3.bSaveFM  = wSaveOld.g3.add('button',undefined,localize(sbSaveFM));
  	wSaveOld.g3.bSaveMIF = wSaveOld.g3.add('button',undefined,localize(sbSaveMIF));
  	wSaveOld.g3.bHelp    = wSaveOld.g3.add('button',undefined,localize(sbHelp));
	wSaveOld.g4 = wSaveOld.add('group',undefined);
  	wSaveOld.g4.alignChildren = ['left',' '];
  	wSaveOld.g4.st3        = wSaveOld.g4.add('statictext',undefined,localize(stHint),{multiline:true});
  	wSaveOld.g4.st3.preferredSize = {width:500, height:60};   // at least 4 lines
	  wSaveOld.g4.st3.graphics.foregroundColor = wSaveOld.g4.st3.graphics.newPen (wSaveOld.g4.st3.graphics.PenType.SOLID_COLOR, [1, 0, 0], 1);
	wSaveOld.g5 = wSaveOld.add('group',undefined);
  	wSaveOld.g5.alignChildren = ['left',' '];
  	wSaveOld.g5.Close    = wSaveOld.g5.add('button',undefined,localize(sbClose));
  	wSaveOld.g5.Cancel   = wSaveOld.g5.add('button',undefined,localize(sbCancel));
// --- set the radio buttons --- depending on the current FM version -----------------------------
  var appVersion = app.VersionMajor;
  var rbuttons   = wSaveOld.g1.rb;
  for (var j = 0; j < rbuttons.children.length; j++) {
    var txt = rbuttons.children[j].text;
    rbuttons.children[j].value = false;
    rbuttons.children[j].visible = true;
    if (j+8 == appVersion ) {
      rbuttons.children[j].value = true;
    }
    if (j+8 > appVersion ) {
//    rbuttons.children[j].visible = false;
      rbuttons.children[j].enabled = false;  // make them inactive
    }
  }
// --- behaviour of edit field
  wSaveOld.g2.folderName.onChange = function () {
    gsFolder = wSaveOld.g2.folderName.text;
    wSaveOld.g4.st3.text = "-----";               // remove the intial hint message
 }
// --- behaviour of checkbox
  wSaveOld.g3.cCurrent.onClick = function () {
    if (wSaveOld.g3.cCurrent.value) {
      path = GetCurrentFolder ();
      wSaveOld.g2.folderName.text = path;
      wSaveOld.g4.st3.text = localize(gsETBsavO_Warning);
    } else {
      wSaveOld.g2.folderName.text = localize(stAdvice);
    wSaveOld.g4.st3.text = "-----";
    }
//  wSaveOld.show();
  }
// --- behaviour of the buttons
  wSaveOld.g2.bBrowse.onClick  = ButtonBrowse;
  wSaveOld.g3.bSaveFM.onClick  = ButtonSaveFM;
  wSaveOld.g3.bSaveMIF.onClick = ButtonSaveMIF;
  wSaveOld.g3.bHelp.onClick    = ButtonHelp;
  wSaveOld.g5.Close.onClick    = function () {
    wSaveOld.close();
  }
  wSaveOld.g5.Cancel.onClick   = function () {
    wSaveOld.close();
  }
// --- final action in dialgue
	wSaveOld.g5.Close.enabled = false;
	wSaveOld.g5.Cancel.enabled = true;
  wSaveOld.show();
} // --- end DialogSaveOld

function ButtonBrowse () { //======================================================================
  var userFolder = new Folder(app.OpenDir);
  var targetFolder = userFolder.selectDlg(localize(gsETBsavO_SelTarget));
  if (targetFolder !== null) {
    wSaveOld.g2.folderName.text = targetFolder.fsName;
  }
} // --- end ButtonBrowse

function ButtonSaveFM () { //======================================================================
//$.bp(true);
  gbETBsavO_MIF = false;
  ButtonSave ();
} // --- end ButtonSaveFM

function ButtonSaveMIF () { //=====================================================================
//$.bp(true);
  gbETBsavO_MIF = true;
  ButtonSave ();
} // --- end ButtonSaveMIF

function ButtonSave () { // =======================================================================
  var folderText = wSaveOld.g2.folderName.text;
  var lastBSlash, fileType;
    if (folderText === "") {                      // Validate the target folder.
    alert (localize(gsETBsavO_Target));
    return;
  } else {                                        // Make sure the folder exists.
    if (Folder(folderText).exists === false) {
      alert (localize(gsETBsavO_NoTarget));
      return;
    } else {
      lastBSlash = folderText.lastIndexOf("\\");
      if (lastBSlash == folderText.length-1) {    // remove terminating \
        folderText = folderText.substring(0, lastBSlash);
      }
      wSaveOld.g2.folderName.text = folderText;
      goETBsavO_Folder = folderText;                      // target folder
    }
  }
  gsETBsavO_FMversion = SelectedRadioButton (wSaveOld.g1.rb); // text from selected radio button
  giETBsavO_FileType = GetFileType (gsETBsavO_FMversion);
// --- for a book
  if (app.ActiveBook.ObjectValid()) {
    goETBsavO_ActiveBook = app.ActiveBook;
    ProcessBook (goETBsavO_ActiveBook.Name);
  }
// --- for single document
  if (app.ActiveDoc.ObjectValid()) {
    goETBsavO_ActiveDoc = app.ActiveDoc;
    ProcessDoc (goETBsavO_ActiveDoc.Name);
  }
  UpdateStatus (localize(gsETBsavO_Success));
} // --- end ButtonSave

function ButtonHelp () {
//$.bp(true);
  OpenWebURL ("http://www.daube.ch/docu/fmaker73.html");
} // --- end ButtonHelp

function SelectedRadioButton (rbuttons) { //=======================================================
var j, sText
  for (j = 0; j < rbuttons.children.length; j++) {
    if (rbuttons.children[j].value == true){
      sText = rbuttons.children[j].text
      return sText.substring(0, 5);
    }
  }
} // --- end SelectedRadioButton
function SetUpMenu() { // =========================================================================
// This function MUST NOT be named DefineMenu - because this is a reserved function!
// Errors in ESTK: Menu position can not be changed - see bug report FRMAKER-2752
// Try to solve via menu configuration in ETB
  var sDocMenu     = { en: "Save document as old version …",
                       de: "Dokument in älterer version speichern …",
                       fr: "Enregistrer document comme version ancienne …"};
  var sDocShortLbl = { en: "ESC q s o",
                       de: "ESC q s o",
                       fr: "ESC q s o"};
  var sDocShortCut = { en: "\\!qso",
                       de: "\\!qso",
                       fr: "\\!qso"};

  var sBookMenu     = { en: "Save book as old version",
                        de: "Buch in älterer version speichern …",
                        fr: "Enregistrer livre comme version ancienne …"};
  var sBookShortLbl = { en: "ESC q s o",
                        de: "ESC q s o",
                        fr: "ESC q s o"};
  var sBookShortCut = { en: "\\!qso",
                        de: "\\!qso",
                        fr: "\\!qso"};
                                                // --- Menu definition for documents
  var menuLocation = app.GetNamedMenu("FileMenu");
  var cmdDoc = menuLocation.DefineAndAddCommand(1, "ETB_docSaveOld", localize(sDocMenu), localize(sDocShortCut));
  cmdDoc.KeyboardShortcutLabel = localize(sDocShortLbl);
//cmdDoc.Description = "Save document as old FM version [ETB]";
                                                  // --- Menu definition for books
  menuLocation = app.GetNamedMenu("BookFileMenu");
  var cmdBook = menuLocation.DefineAndAddCommand(1, "ETB_bookSaveOld", localize(sBookMenu), localize(sBookShortCut));
  cmdBook.KeyboardShortcutLabel = localize(sBookShortLbl);
//cmdBook.Description = "Save book as old FM version [ETB]";
  UpdateMenus();

} // --- end SetUpMenu

function Command(cmd) { // ========================================================================
	switch(cmd)  {	                                // Respond to menu command.
		case 1:
      DialogSaveOld();
      break;
    }
} // --- end Command

function UpdateStatus (sStatus) { // ==============================================================
  wSaveOld.g5.Close.enabled = true;
  wSaveOld.g5.Cancel.enabled = false;
  wSaveOld.g4.st3.text = sStatus;
} // --- end UpdateStatus

function GetCurrentFolder () { // =================================================================
  oDoc = app.ActiveDoc;
  if (oDoc.ObjectValid()) {
    path = oDoc.Name;
  } else {
    oBook = app.ActiveBook;
      if (oDoc.ObjectValid()) {
      path = oBook.Name;
    }
  }
  lastBSlash = path.lastIndexOf("\\");            // \ needs escaping
  folder = path.substring(0, lastBSlash);
  return folder
} // --- end GetCurrentFolder

function ProcessDoc (name) { // ===================================================================
  var oDoc;
  oDoc = GetDocOrBook (name);                     // Open the document.
  if (oDoc.obj) {
    SaveAsFileType (oDoc.obj, goETBsavO_Folder, giETBsavO_FileType); // Save and close the document.
    if (oDoc.isOpen === false) {
      oDoc.obj.Close (true);
    }
  }
} // --- end ProcessDoc

function ProcessBook (name) { // ==================================================================
  var book = GetDocOrBook (name);                 // Open the book.
  var bookComp = book.obj.FirstComponentInBook;
  while (bookComp.ObjectValid()) {
    if (bookComp.BookComponentFileType === Constants.FV_BK_FM) { // Document
      ProcessDoc (bookComp.Name);
    }
    bookComp = bookComp.NextBookComponentInDFSOrder;
  }
// update the book (component paths); save the book and close it.
  UpdateComponentPaths (book.obj, goETBsavO_Folder, giETBsavO_FileType);
  SaveAsFileType (book.obj, goETBsavO_Folder, giETBsavO_FileType);
  if (book.isOpen === false) {
    book.obj.Close (true);
  }
} // --- end ProcessBook

function ProcessComponents (book) { // ============================================================
  var bookComp = book.FirstComponentInBook;
  while (bookComp.ObjectValid()) {
    if (bookComp.BookComponentFileType === Constants.FV_BK_FM) { // Document
      ProcessDoc (bookComp.Name);
    }
    bookComp = bookComp.NextBookComponentInDFSOrder;
  }
} // --- end ProcessComponents

function GetFileType (fmVersion) { // =============================================================
  if (gbETBsavO_MIF) {
        giETBsavO_FileType =  Constants.FV_SaveFmtInterchange;
  } else {
    switch (fmVersion) {
      case "FM-08":
        giETBsavO_FileType =  Constants.FV_SaveFmtBinary80; break;
      case "FM-09":
        giETBsavO_FileType = Constants.FV_SaveFmtBinary90; break;
      case "FM-10":
        giETBsavO_FileType = Constants.FV_SaveFmtBinary100; break;
      case "FM-11":
        giETBsavO_FileType = Constants.FV_SaveFmtBinary110; break;
      case "FM-12":
        giETBsavO_FileType = Constants.FV_SaveFmtBinary120; break;
      case "FM-13":
        giETBsavO_FileType = Constants.FV_SaveFmtBinary130; break;
      case "FM-14":
        giETBsavO_FileType = Constants.FV_SaveFmtBinary140; break;
      case "FM-15":
        giETBsavO_FileType = Constants.FV_SaveFmtBinary150; break;
      case "FM-16":
        giETBsavO_FileType = Constants.FV_SaveFmtBinary160; break;
      case "FM-17":
        giETBsavO_FileType = Constants.FV_SaveFmtBinary170; break;
      default:
        giETBsavO_FileType = GetDefaultFileType();
    }
  }
  return giETBsavO_FileType;                              // return also in function
} // --- end GetFileType

function GetDefaultFileType () { // ===============================================================
  var appVersion = app.VersionMajor;
  if (gbETBsavO_MIF) {
        giETBsavO_FileType =  Constants.FV_SaveFmtInterchange;
  } else {
    switch (appVersion) {
      case 10:
        case "FM-10":
          giETBsavO_FileType = Constants.FV_SaveFmtBinary100; break;
        case "FM-11":
          giETBsavO_FileType = Constants.FV_SaveFmtBinary110; break;
        case "FM-12":
          giETBsavO_FileType = Constants.FV_SaveFmtBinary120; break;
        case "FM-13":
          giETBsavO_FileType = Constants.FV_SaveFmtBinary130; break;
        case "FM-14":
          giETBsavO_FileType = Constants.FV_SaveFmtBinary140; break;
        case "FM-15":
          giETBsavO_FileType = Constants.FV_SaveFmtBinary150; break;
        case "FM-16":
          giETBsavO_FileType = Constants.FV_SaveFmtBinary160; break;
        case "FM-17":
          giETBsavO_FileType = Constants.FV_SaveFmtBinary170; break;
      default:
        alert (localize(gsETBsavO_UnknownFM));
        exit;
    }
  }
} // --- end GetDefaultFileType

function OpenDocOrBook (filename) { // ============================================================
  var i = 0, doc = 0;
  var docOrBook = {obj: 0, isOpen: false};        // Make an object to return.
  var openProps = GetOpenDefaultParams();         // Get default property list for opening documents or books.
  var retParm = new PropVals();                   // Get a property list to return any error messages.
                                                  // Set specific open property values to open the document or book.
  i=GetPropIndex(openProps,Constants.FS_AlertUserAboutFailure);
  openProps[i].propVal.ival=false;
  i=GetPropIndex(openProps,Constants.FS_MakeVisible);
  openProps[i].propVal.ival=false;
  i=GetPropIndex(openProps,Constants.FS_FileIsOldVersion);
  openProps[i].propVal.ival=Constants.FV_DoOK;
  i=GetPropIndex(openProps,Constants.FS_FileIsInUse);
  openProps[i].propVal.ival=Constants.FV_ResetLockAndContinue;
  i=GetPropIndex(openProps,Constants.FS_FontChangedMetric);
  openProps[i].propVal.ival=Constants.FV_DoOK;
  i=GetPropIndex(openProps,Constants.FS_FontNotFoundInCatalog);
  openProps[i].propVal.ival=Constants.FV_DoOK;
  i=GetPropIndex(openProps,Constants.FS_FontNotFoundInDoc);
  openProps[i].propVal.ival=Constants.FV_DoOK;
  i=GetPropIndex(openProps,Constants.FS_RefFileNotFound);
  openProps[i].propVal.ival=Constants.FV_AllowAllRefFilesUnFindable;

  doc = Open(filename,openProps,retParm);         // Attempt to open the document or book.
  if (doc.ObjectValid() === 1) {
      docOrBook.obj = doc;
  } else {                                        // If the document can't be open, print the errors to the Console.
      PrintOpenStatus(retParm);
  }
  return docOrBook;                               // Return the document or book object.
} // --- end OpenDocOrBook

function GetDocOrBook (filename) { // =============================================================
  if (/\.fm$/i.test (filename) === true) {        // See if the document or book is already open.
    var docOrBook = DocIsOpen (filename);         // Doc
  } else {                                        // Book
    var docOrBook = BookIsOpen (filename);
  }

  if (docOrBook.obj) {
    return docOrBook;
  } else {                                        // The document is not already open, so open it.
    return OpenDocOrBook (filename);
  }
} // --- end GetDocOrBook

function DocIsOpen (filename) { // ================================================================
  var file = File(filename);                      // Make a File object from the file name.
  var fileName = file.fullName.toUpperCase();     // Uppercase the filename for easy comparison.
  var doc = {obj: 0, isOpen: false};              // Make an object to return.
  var currDoc = app.FirstOpenDoc;                 // Loop through the open documents in the session.
  while (currDoc.id) {              // Compare the document’s name with the one we are looking for.
    if (File(currDoc.Name).fullName.toUpperCase() === fileName) { // The document we are looking for is open.
      doc.obj = currDoc;                          // Add it to the object and return it.
      doc.isOpen = true;
      return doc;
    }
    currDoc = currDoc.NextOpenDocInSession;
  }
  return doc;                                     // Document is not open; return the “empty” object.
} // --- end DocIsOpen

function BookIsOpen (filename) { // ===============================================================
  var file = File(filename);                      // Make a File object from the file name.
  var fileName = file.fullName.toUpperCase();     // Uppercase the filename for easy comparison.
  var book = {obj: 0, isOpen: false};             // Make an object to return.
  var currBook = app.FirstOpenBook;               // Loop through the open books in the session.

  while (currBook.id) {             // Compare the document’s name with the one we are looking for.
    if (File(currBook.Name).fullName.toUpperCase() === fileName) { // The document we are looking for is open.
      book.obj = currBook;                        // Add it to the object and return it.
      book.isOpen = true;
      return book;
    }
    currBook = currBook.NextOpenBookInSession;
  }
  return book;                                    // Book is not open; return the “empty” object.
} // --- end BookIsOpen

function UpdateComponentPaths (book, folder, fileType) { // =======================================
  var bookComp = book.FirstComponentInBook;

  while (bookComp.ObjectValid()) {
    if (bookComp.BookComponentFileType === Constants.FV_BK_FM) { // Document
      bookComp.Name = folder +  /\\[^\\]+$/.exec (bookComp.Name)[0];
//  } else if (bookComp.BookComponentFileType === Constants.FV_BK_BOOK) {
//    bookComp.Name = folder +  /\\[^\\]+$/.exec (bookComp.Name)[0];
    }
    bookComp = bookComp.NextBookComponentInDFSOrder;
  }
  SaveAsFileType (book, folder, fileType);
} // --- end UpdateComponentPaths

function SaveAsFileType (docOrBook, folder, fileType) { // ========================================
  var params, returnParams, k, fullName, baseName, lastBSlash, lastDot, savePath, extension;

//$.bp(true);
  params = GetSaveDefaultParams();
  returnParams = new PropVals();
  k = GetPropIndex (params, Constants.FS_FileType);
  params[k].propVal.ival = fileType;
  fullName = docOrBook.Name;
  lastDot    = fullName.lastIndexOf( "." ) ;      // .fm or .book
  lastBSlash = fullName.lastIndexOf("\\");
  baseName = fullName.substr(lastBSlash+1, lastDot-lastBSlash-1);
  extension  = fullName.substr(lastDot);          // .book or .fm
  if (gbETBsavO_MIF) {extension = ".mif";}
  savePath = folder + "\\" + baseName + extension;
  docOrBook.Save (savePath, params, returnParams);
	if (FA_errno !== 0) {
		PrintSaveStatus (returnParams);
	}
} // --- end SaveAsFileType

function OpenWebURL (URL) { // ====================================================================
  var path = GetScriptPath () + "\\";
// Create an executable (batch, shortcut) file and execute it
  var tempFile = File (path + "WebShortCut.url");
  tempFile.open("w");
  tempFile.encoding = "UTF-8";
  tempFile.writeln ("[{000214A0-0000-0000-C000-000000000046}]");
  tempFile.writeln ("Prop3=19,2");
  tempFile.writeln ("[InternetShortcut]");
  tempFile.writeln ("URL=" + URL);
  tempFile.close();
  tempFile.execute ();
  tempFile.delete;
}

function GetScriptPath () { //=====================================================================
// Returns path in the function
  var scrPath, winPath, lastBSlash;

// $.fileName: script path (including the file-name of the script)
// platform independent format e.g. /e/_DDDprojects/FM+EN-escript/escript
  scrPath = $.fileName;
  winPath = new File (scrPath);
  scrPath = winPath.fsName;                       // get Windows format
  lastBSlash = scrPath.lastIndexOf("\\");         // \ needs escaping
  scrPath = scrPath.substring(0, lastBSlash);
  return scrPath;                                 // no final \ !
} // --- end GetScriptPath

function SwitchUILanguage () {; // === Switch language according to application language ============
  switch (app.Language) {
    case Constants.FV_LANG_BRITISH:
    case Constants.FV_LANG_ENGLISH: {
      $.locale = "en"; break;
    }
    case Constants.FV_LANG_CANADIAN_FRENCH:
    case Constants.FV_LANG_FRENCH: {
      $.locale = "fr"; break;
    }
    case Constants.FV_LANG_GERMAN:
    case Constants.FV_LANG_NEW_GERMAN:
    case Constants.FV_LANG_NEW_SWISS_GERMAN:
    case Constants.FV_LANG_SWISS_GERMAN:	{
      $.locale = "de"; break;
    }
    default: {
      $.locale = "en"; break;
    }
  }
} // --- end SwitchUILanguage
